@echo off

:: This file is intended to be called from the target's build directory. All
:: source paths used here are relative to that directory.

set NASM_LOCATION="C:\Program Files (x86)\NASM\nasm.exe"
set WINRAR_LOCATION="C:\Program Files\WinRAR\WinRAR.exe"

set COMMON_LOCATION=..\..\..\_common\patch\build

for /d %%i in (..\..\.) do set BUILD_FRIENDLY_NAME=%%~nxi
for /f "usebackq tokens=1,2 delims==" %%i in (`wmic os get LocalDateTime /value 2^>nul`) do if /%%i/==/LocalDateTime/ set BUILD_DATE=%%j
set BUILD_DATE=%BUILD_DATE:~0,8%

if exist %NASM_LOCATION% (set BUILD_SOURCES=1) else (set BUILD_SOURCES=0)
if exist %WINRAR_LOCATION% (set CREATE_ARCHIVE=1) else (set CREATE_ARCHIVE=0)

echo.
echo ===============================================================================
echo Cleaning up
echo ===============================================================================
echo.
if %BUILD_SOURCES%==1 del bin\* /Q
del work\* /Q
del ..\%BUILD_ARCHIVE_PREFIX%_*.zip

if %BUILD_SOURCES%==1 (
    echo.
    echo ===============================================================================
    echo Building awardext.asm
    echo ===============================================================================
    echo.
    %NASM_LOCATION% -d%BUILD_TARGET% %COMMON_LOCATION%\source\awardext.asm -isource -i%COMMON_LOCATION%\source -obin\awardext
    if errorlevel 1 goto exit

    echo.
    echo ===============================================================================
    echo Updating awardext.equ
    echo ===============================================================================
    echo.
    %COMMON_LOCATION%\tool\MapToEqu.exe bin\awardext.map source\awardext.equ
    if errorlevel 1 goto exit

    echo.
    echo ===============================================================================
    echo Building mainE000.asm
    echo ===============================================================================
    echo.
    %NASM_LOCATION% -d%BUILD_TARGET% %COMMON_LOCATION%\source\mainE000.asm -isource -i%COMMON_LOCATION%\source -obin\mainE000
    if errorlevel 1 goto exit

    echo.
    echo ===============================================================================
    echo Building mainF000.asm
    echo ===============================================================================
    echo.
    %NASM_LOCATION% -d%BUILD_TARGET% %COMMON_LOCATION%\source\mainF000.asm -isource -i%COMMON_LOCATION%\source -obin\mainF000
    if errorlevel 1 goto exit

    echo.
    echo ===============================================================================
    echo Building boot.asm
    echo ===============================================================================
    echo.
    %NASM_LOCATION% -d%BUILD_TARGET% %COMMON_LOCATION%\source\boot.asm -isource -i%COMMON_LOCATION%\source -obin\boot
    if errorlevel 1 goto exit
)

echo.
echo ===============================================================================
echo Concatenating microcodes
echo ===============================================================================
echo.
%COMMON_LOCATION%\tool\GenASCPU.exe work\cpucode.exe microcode\*.bin
if errorlevel 1 goto exit

echo.
echo ===============================================================================
echo Compressing microcodes
echo ===============================================================================
echo.
%COMMON_LOCATION%\tool\lha.exe a work\temp.lzh -o2 -h1 work\cpucode.exe
if errorlevel 1 goto exit
del work\cpucode.exe
ren work\temp.lzh cpucode.exe

echo.
echo ===============================================================================
echo Fixing microcode module
echo ===============================================================================
echo.
%COMMON_LOCATION%\tool\ApplyPat.exe patch\module_cpucode.pat work\cpucode.exe
if errorlevel 1 goto exit

echo.
echo ===============================================================================
echo Decompressing original system ROM
echo ===============================================================================
echo.
%COMMON_LOCATION%\tool\lha.exe e original\%BUILD_SYSTEM_ROM% work\
if errorlevel 1 goto exit

echo.
echo ===============================================================================
echo Patching system ROM (first half)
echo ===============================================================================
echo.
%COMMON_LOCATION%\tool\ApplyPat.exe patch\mainE000.pat work\%BUILD_SYSTEM_ROM%
if errorlevel 1 goto exit

echo.
echo ===============================================================================
echo Patching system ROM (second half)
echo ===============================================================================
echo.
%COMMON_LOCATION%\tool\ApplyPat.exe patch\mainF000.pat work\%BUILD_SYSTEM_ROM%
if errorlevel 1 goto exit

echo.
echo ===============================================================================
echo Compressing system ROM
echo ===============================================================================
echo.
%COMMON_LOCATION%\tool\lha.exe a work\temp.lzh -o2 -h1 work\%BUILD_SYSTEM_ROM%
if errorlevel 1 goto exit
del work\%BUILD_SYSTEM_ROM%
ren work\temp.lzh %BUILD_SYSTEM_ROM%

echo.
echo ===============================================================================
echo Fixing system ROM module
echo ===============================================================================
echo.
%COMMON_LOCATION%\tool\ApplyPat.exe patch\module_system.pat work\%BUILD_SYSTEM_ROM%
if errorlevel 1 goto exit

echo.
echo ===============================================================================
echo Decompressing extension ROM
echo ===============================================================================
echo.
%COMMON_LOCATION%\tool\lha.exe e original\AWARDEXT.ROM work\
if errorlevel 1 goto exit

echo.
echo ===============================================================================
echo Patching extension ROM
echo ===============================================================================
echo.
%COMMON_LOCATION%\tool\ApplyPat.exe patch\awardext.pat work\AWARDEXT.ROM
if errorlevel 1 goto exit

echo.
echo ===============================================================================
echo Compressing extension ROM
echo ===============================================================================
echo.
%COMMON_LOCATION%\tool\lha.exe a work\temp.lzh -o2 -h1 work\AWARDEXT.ROM
if errorlevel 1 goto exit
del work\AWARDEXT.ROM
ren work\temp.lzh AWARDEXT.ROM

echo.
echo ===============================================================================
echo Fixing extension ROM module
echo ===============================================================================
echo.
%COMMON_LOCATION%\tool\ApplyPat.exe patch\module_awardext.pat work\AWARDEXT.ROM
if errorlevel 1 goto exit

echo.
echo ===============================================================================
echo Reassembling and fixing BIOS image
echo ===============================================================================
echo.
copy original\%BUILD_IMAGE% work>nul
if errorlevel 1 goto exit
%COMMON_LOCATION%\tool\ApplyPat.exe patch\image.pat work\%BUILD_IMAGE%
if errorlevel 1 goto exit

echo.
echo ===============================================================================
echo Cleaning up
echo ===============================================================================
echo.
copy work\%BUILD_IMAGE% .. /Y>nul
if errorlevel 1 goto exit
del work\* /Q

if %CREATE_ARCHIVE%==1 (
    echo.
    echo ===============================================================================
    echo Building archive
    echo ===============================================================================
    echo.
    %WINRAR_LOCATION% a ..\%BUILD_ARCHIVE_PREFIX%_%BUILD_DATE%.zip ..\%BUILD_IMAGE%
    %WINRAR_LOCATION% a ..\%BUILD_ARCHIVE_PREFIX%_%BUILD_DATE%.zip -ap"sources\%BUILD_FRIENDLY_NAME%\patch" ..\build
    %WINRAR_LOCATION% a ..\%BUILD_ARCHIVE_PREFIX%_%BUILD_DATE%.zip -ap"sources" %COMMON_LOCATION%
)

:exit

set NASM_LOCATION=
set WINRAR_LOCATION=

set COMMON_LOCATION=

set BUILD_FRIENDLY_NAME=
set BUILD_DATE=

set BUILD_SOURCES=
set CREATE_ARCHIVE=
