;****************************************************************************
; BIOS patch code for
; - ASUS CUBX 1008 beta 004 (1008cu.004)
; - ASUS CUBX-L/E 1008 beta 004 (1008cul.004)
; - ASUS CUV4X 1010 beta 003 (1010.003)
; - ASUS CUV4X-CME 1002 beta 002 (1002.002)
; - ASUS CUV4X-D 1016 (CUVD1016.AWD)
; - ASUS CUV4X-DLS 1016 (VDLS1016.AWD)
; - ASUS CUV4X-E (onboard audio) 1006 beta 005 (1006.005)
; - ASUS CUV4X-E (no onboard audio) 1005 beta 005 (1005.005)
; - ASUS CUV4X-M 1008 beta 003 (1008.003)
; - ASUS P3B-F 1008 beta 004 (1008f.004)
; - ASUS P3C-E (no audio/onboard audio) 1025 beta 002 (1025e.002)
; - ASUS P3C-E (software AC97) 1025 beta 002 (1025e_swa.002)
; - ASUS P3V133 1003 beta 002 (1003.002)
; - ASUS P3V4X 1006 beta 004 (1006.004)
; - ASUS TUV4X 1006 beta 002 (1006.002)
;****************************************************************************
;
; EXTGROUP file containing new code and data to be added to the BIOS
;
;****************************************************************************
;
; Changes:
; - support of multipliers 12.5x through 16.0x on the "Advanced" tab of the
;   BIOS Setup Utility, including display of suggested CPU speeds for
;   standard FSB speeds
; - fixed display of multiplier 2.0x for processors with Klamath core
; - matched selection of multipliers for unlocked processors to core types
; - added Setup Utility "Enable" option for Symbios SCSI BIOS to force
;   loading the ROM module even if no matching adapter is present
; - optional: added CPU speeds based on 133 MHz FSB as default options for
;   440BX-based boards (to be enabled in macro.def)
; - disabled CPU temperature display for Via CPUs
; - disabled -5 V display for modern PSUs without -5 V rail
;
;****************************************************************************

;----------------------------------------------------------------------------
; assembler directives/options
;----------------------------------------------------------------------------

[map symbols bin\awardext.map]
bits 16
cpu 686

;----------------------------------------------------------------------------
; includes
;----------------------------------------------------------------------------

%include	"macro.def"		; macro definitions
%include	"bios.equ"		; addresses of BIOS code and data
					; used in patch routines

;----------------------------------------------------------------------------
; origin
;----------------------------------------------------------------------------

patchStart:
	org 	patchLocEXT		; go to free location in EXTGROUP ROM
					; to place patch data and code into

;----------------------------------------------------------------------------
; data
;----------------------------------------------------------------------------

%ifdef PATCH_KLAMATH_MULT
KlamathSpeedTab:
	db	"*133MHz", 0
	db	"*200MHz", 0
%ifdef FSB133_SUPPORT
	db	"*266MHz", 0
%endif					; FSB133_SUPPORT
	db	" 166MHz", 0
	db	" 250MHz", 0
%ifdef FSB133_SUPPORT
	db	" 333MHz", 0
%endif					; FSB133_SUPPORT
	db	" 200MHz", 0
	db	" 300MHz", 0
%ifdef FSB133_SUPPORT
	db	" 400MHz", 0
%endif					; FSB133_SUPPORT
	db	" 233MHz", 0
	db	" 350MHz", 0
%ifdef FSB133_SUPPORT
	db	" 466MHz", 0
%endif					; FSB133_SUPPORT
	db	" 266MHz", 0
	db	" 400MHz", 0
%ifdef FSB133_SUPPORT
	db	" 533MHz", 0
%endif					; FSB133_SUPPORT
	db	" 300MHz", 0
	db	" 450MHz", 0
%ifdef FSB133_SUPPORT
	db	" 600MHz", 0
%endif					; FSB133_SUPPORT
	db	" 133MHz", 0
	db	" 200MHz", 0
%ifdef FSB133_SUPPORT
	db	" 266MHz", 0
%endif					; FSB133_SUPPORT
	db	"       ", 0
	db	"       ", 0
%ifdef FSB133_SUPPORT
	db	"       ", 0
%endif					; FSB133_SUPPORT
	db	"       ", 0
	db	"       ", 0
%ifdef FSB133_SUPPORT
	db	"       ", 0
%endif					; FSB133_SUPPORT
	db	"       ", 0
	db	"       ", 0
%ifdef FSB133_SUPPORT
	db	"       ", 0
%endif					; FSB133_SUPPORT
	db	"       ", 0
	db	"       ", 0
%ifdef FSB133_SUPPORT
	db	"       ", 0
%endif					; FSB133_SUPPORT
	db	"       ", 0
	db	"       ", 0
%ifdef FSB133_SUPPORT
	db	"       ", 0
%endif					; FSB133_SUPPORT
	db	"       ", 0
	db	"       ", 0
%ifdef FSB133_SUPPORT
	db	"       ", 0
%endif					; FSB133_SUPPORT
	db	"       ", 0
	db	"       ", 0
%ifdef FSB133_SUPPORT
	db	"       ", 0
%endif					; FSB133_SUPPORT
	db	"       ", 0
	db	"       ", 0
%ifdef FSB133_SUPPORT
	db	"       ", 0
%endif					; FSB133_SUPPORT
	db	"       ", 0
	db	"       ", 0
%ifdef FSB133_SUPPORT
	db	"       ", 0
%endif					; FSB133_SUPPORT
%endif					; PATCH_KLAMATH_MULT

%ifdef PATCH_SPEED_TAB
SpeedTab:
	db	" 133MHz", 0
	db	" 200MHz", 0
%ifdef FSB133_SUPPORT
	db	" 266MHz", 0
%endif					; FSB133_SUPPORT
	db	" 166MHz", 0
	db	" 250MHz", 0
%ifdef FSB133_SUPPORT
	db	" 333MHz", 0
%endif					; FSB133_SUPPORT
	db	" 200MHz", 0
	db	" 300MHz", 0
%ifdef FSB133_SUPPORT
	db	" 400MHz", 0
%endif					; FSB133_SUPPORT
	db	" 233MHz", 0
	db	" 350MHz", 0
%ifdef FSB133_SUPPORT
	db	" 466MHz", 0
%endif					; FSB133_SUPPORT
	db	" 266MHz", 0
	db	" 400MHz", 0
%ifdef FSB133_SUPPORT
	db	" 533MHz", 0
%endif					; FSB133_SUPPORT
	db	" 300MHz", 0
	db	" 450MHz", 0
%ifdef FSB133_SUPPORT
	db	" 600MHz", 0
%endif					; FSB133_SUPPORT
	db	" 333MHz", 0
	db	" 500MHz", 0
%ifdef FSB133_SUPPORT
	db	" 667MHz", 0
%endif					; FSB133_SUPPORT
	db	" 366MHz", 0
	db	" 550MHz", 0
%ifdef FSB133_SUPPORT
	db	" 733MHz", 0
%endif					; FSB133_SUPPORT
	db	" 400MHz", 0
	db	" 600MHz", 0
%ifdef FSB133_SUPPORT
	db	" 800MHz", 0
%endif					; FSB133_SUPPORT
	db	" 433MHz", 0
	db	" 650MHz", 0
%ifdef FSB133_SUPPORT
	db	" 866MHz", 0
%endif					; FSB133_SUPPORT
	db	" 466MHz", 0
	db	" 700MHz", 0
%ifdef FSB133_SUPPORT
	db	" 933MHz", 0
%endif					; FSB133_SUPPORT
	db	" 500MHz", 0
	db	" 750MHz", 0
%ifdef FSB133_SUPPORT
	db	"1000MHz", 0
%endif					; FSB133_SUPPORT
	db	" 533MHz", 0
	db	" 800MHz", 0
%ifdef FSB133_SUPPORT
	db	"1066MHz", 0
%endif					; FSB133_SUPPORT
	db	"       ", 0
	db	"       ", 0
%ifdef FSB133_SUPPORT
	db	"       ", 0
%endif					; FSB133_SUPPORT
	db	"       ", 0
	db	"       ", 0
%ifdef FSB133_SUPPORT
	db	"       ", 0
%endif					; FSB133_SUPPORT
	db	"       ", 0
	db	"       ", 0
%ifdef FSB133_SUPPORT
	db	"       ", 0
%endif					; FSB133_SUPPORT
	db	" 566MHz", 0
	db	" 850MHz", 0
%ifdef FSB133_SUPPORT
	db	"1133MHz", 0
%endif					; FSB133_SUPPORT
	db	" 600MHz", 0
	db	" 900MHz", 0
%ifdef FSB133_SUPPORT
	db	"1200MHz", 0
%endif					; FSB133_SUPPORT
	db	" 633MHz", 0
	db	" 950MHz", 0
%ifdef FSB133_SUPPORT
	db	"1266MHz", 0
%endif					; FSB133_SUPPORT
	db	" 667MHz", 0
	db	"1000MHz", 0
%ifdef FSB133_SUPPORT
	db	"1333MHz", 0
%endif					; FSB133_SUPPORT
	db	" 700MHz", 0
	db	"1050MHz", 0
%ifdef FSB133_SUPPORT
	db	"1400MHz", 0
%endif					; FSB133_SUPPORT
	db	" 733MHz", 0
	db	"1100MHz", 0
%ifdef FSB133_SUPPORT
	db	"1466MHz", 0
%endif					; FSB133_SUPPORT
	db	" 766MHz", 0
	db	"1150MHz", 0
%ifdef FSB133_SUPPORT
	db	"1533MHz", 0
%endif					; FSB133_SUPPORT
	db	" 800MHz", 0
	db	"1200MHz", 0
%ifdef FSB133_SUPPORT
	db	"1600MHz", 0
%endif					; FSB133_SUPPORT
	db	" 866MHz", 0
	db	"1300MHz", 0
%ifdef FSB133_SUPPORT
	db	"1733MHz", 0
%endif					; FSB133_SUPPORT
	db	" 933MHz", 0
	db	"1400MHz", 0
%ifdef FSB133_SUPPORT
	db	"1866MHz", 0
%endif					; FSB133_SUPPORT
	db	"1000MHz", 0
	db	"1500MHz", 0
%ifdef FSB133_SUPPORT
	db	"2000MHz", 0
%endif					; FSB133_SUPPORT
	db	"1066MHz", 0
	db	"1600MHz", 0
%ifdef FSB133_SUPPORT
	db	"2133MHz", 0
%endif					; FSB133_SUPPORT
	db	" 833MHz", 0
	db	"1250MHz", 0
%ifdef FSB133_SUPPORT
	db	"1666MHz", 0
%endif					; FSB133_SUPPORT
	db	" 900MHz", 0
	db	"1350MHz", 0
%ifdef FSB133_SUPPORT
	db	"1800MHz", 0
%endif					; FSB133_SUPPORT
	db	" 966MHz", 0
	db	"1450MHz", 0
%ifdef FSB133_SUPPORT
	db	"1933MHz", 0
%endif					; FSB133_SUPPORT
	db	"1033MHz", 0
	db	"1550MHz", 0
%ifdef FSB133_SUPPORT
	db	"2066MHz", 0
%endif					; FSB133_SUPPORT
%endif					; PATCH_SPEED_TAB

%ifdef FSB133_OPTIONAL
%ifdef NEW_SPEED_LOCKED_TAB
SpeedLockedTab:
	db	" Manual", 0
SpeedLockedEntr:
	db	"", 0
	db	"", 0
%ifdef FSB133_SUPPORT
	db	"", 0
%endif					; FSB133_SUPPORT
	times 27	db	1
%ifdef FSB133_SUPPORT
	times 6		db	1
%endif					; FSB133_SUPPORT
%else					; NEW_SPEED_LOCKED_TAB
	SpeedLockedTab	equ	spdLockTabOrig
%endif					; NEW_SPEED_LOCKED_TAB
%endif					; FSB133_OPTIONAL

%ifdef NEW_SPEED_ROUTINES
%ifndef NEW_SPEED_LOCKED_TAB
	SpeedLockedEntr	equ	spdLockEntrOrig
%endif					; NEW_SPEED_LOCKED_TAB
%endif					; NEW_SPEED_ROUTINES

%ifdef PATCH_KLAMATH_MULT
KlamathMultTab:
	db	"*2.0x", 0
	db	" 2.5x", 0
	db	" 3.0x", 0
	db	" 3.5x", 0
	db	" 4.0x", 0
	db	" 4.5x", 0
	db	" 2.0x", 0
	db	" ", 1
	db	" ", 1
	db	" ", 1
	db	" ", 1
	db	" ", 1
	db	" ", 1
	db	" ", 1
	db	" ", 1
	db	" ", 1
%endif					; PATCH_KLAMATH_MULT

%ifdef PATCH_MULT_TAB
MultiplierTab:
	db	" 8.5x", 0
	db	" 9.0x", 0
	db	" 9.5x", 0
	db	"10.0x", 0
	db	"10.5x", 0
	db	"11.0x", 0
	db	"11.5x", 0
	db	"12.0x", 0
	db	"13.0x", 0
	db	"14.0x", 0
	db	"15.0x", 0
	db	"16.0x", 0
	db	"12.5x", 0
	db	"13.5x", 0
	db	"14.5x", 0
	db	"15.5x", 0
%endif					; PATCH_MULT_TAB

%ifdef PATCH_KLAMATH_MULT
KlamathUSpTab:
	db	"Manual", 0
	db	"133MHz", 0		; 2.0 x 66
	db	"166MHz", 0		; 2.5 x 66
	db	"200MHz", 0		; 3.0 x 66
	db	"233MHz", 0		; 3.5 x 66
	db	"266MHz", 0		; 4.0 x 66
	db	"300MHz", 0		; 4.5 x 66
	db	"200MHz", 0		; 2.0 x 100
	db	"250MHz", 0		; 2.5 x 100
	db	"300MHz", 0		; 3.0 x 100
	db	"350MHz", 0		; 3.5 x 100
	db	"400MHz", 0		; 4.0 x 100
	db	"450MHz", 0		; 4.5 x 100
	db	1			; empty/hidden
	db	1			; empty/hidden
	db	1			; empty/hidden

DeschutesUSpTab:
	db	"Manual", 0
	db	"133MHz", 0		; 2.0 x 66
	db	"166MHz", 0		; 2.5 x 66
	db	"200MHz", 0		; 3.0 x 66
	db	"233MHz", 0		; 3.5 x 66
	db	"266MHz", 0		; 4.0 x 66
	db	"300MHz", 0		; 4.5 x 66
	db	"333MHz", 0		; 5.0 x 66
	db	"200MHz", 0		; 2.0 x 100
	db	"250MHz", 0		; 2.5 x 100
	db	"300MHz", 0		; 3.0 x 100
	db	"350MHz", 0		; 3.5 x 100
	db	"400MHz", 0		; 4.0 x 100
	db	"450MHz", 0		; 4.5 x 100
	db	"500MHz", 0		; 5.0 x 100
	db	1			; empty/hidden

MendocinoUSpTab:
	db	"Manual", 0
	db	"300MHz", 0		; 4.5 x 66
	db	"333MHz", 0		; 5.0 x 66
	db	"366MHz", 0		; 5.5 x 66
	db	"400MHz", 0		; 6.0 x 66
	db	"433MHz", 0		; 6.5 x 66
	db	"466MHz", 0		; 7.0 x 66
	db	"500MHz", 0		; 7.5 x 66
	db	"533MHz", 0		; 8.0 x 66
	db	"450MHz", 0		; 4.5 x 100
	db	"500MHz", 0		; 5.0 x 100
	db	"550MHz", 0		; 5.5 x 100
	db	"600MHz", 0		; 6.0 x 100
	db	"650MHz", 0		; 6.5 x 100
	db	"700MHz", 0		; 7.0 x 100
	db	"750MHz", 0		; 7.5 x 100

%ifdef FSB133_NATIVE
UnlockedSpdTab:
	db	" Manual", 0
	db	" 450MHz", 0		; 4.5 x 100
	db	" 500MHz", 0		; 5.0 x 100
	db	" 550MHz", 0		; 5.5 x 100
	db	" 600MHz", 0		; 6.0 x 100
	db	" 650MHz", 0		; 6.5 x 100
	db	" 700MHz", 0		; 7.0 x 100
	db	" 750MHz", 0		; 7.5 x 100
	db	" 800MHz", 0		; 8.0 x 100
	db	" 600MHz", 0		; 4.5 x 133
	db	" 667MHz", 0		; 5.0 x 133
	db	" 733MHz", 0		; 5.5 x 133
	db	" 800MHz", 0		; 6.0 x 133
	db	" 866MHz", 0		; 6.5 x 133
	db	" 933MHz", 0		; 7.0 x 133
	db	"1000MHz", 0		; 7.5 x 133
%else					; FSB133_NATIVE
	UnlockedSpdTab	equ	MendocinoUSpTab
%endif					; FSB133_NATIVE

KlamathUnlCMOSTab:
	db	0			; Manual
	db	36h			; 2.0 x 66
	db	31h			; 2.5 x 66
	db	32h			; 3.0 x 66
	db	33h			; 3.5 x 66
	db	34h			; 4.0 x 66
	db	35h			; 4.5 x 66
	db	76h			; 2.0 x 100
	db	71h			; 2.5 x 100
	db	72h			; 3.0 x 100
	db	73h			; 3.5 x 100
	db	74h			; 4.0 x 100
	db	75h			; 4.5 x 100
	db	0
	db	0
	db	0

DeschutesUnlCMOSTab:
	db	0			; Manual
	db	30h			; 2.0 x 66
	db	31h			; 2.5 x 66
	db	32h			; 3.0 x 66
	db	33h			; 3.5 x 66
	db	34h			; 4.0 x 66
	db	35h			; 4.5 x 66
	db	36h			; 5.0 x 66
	db	70h			; 2.0 x 100
	db	71h			; 2.5 x 100
	db	72h			; 3.0 x 100
	db	73h			; 3.5 x 100
	db	74h			; 4.0 x 100
	db	75h			; 4.5 x 100
	db	76h			; 5.0 x 100
	db	0

MendocinoUnlCMOSTab:
	db	0			; Manual
	db	35h			; 4.5 x 66
	db	36h			; 5.0 x 66
	db	37h			; 5.5 x 66
	db	38h			; 6.0 x 66
	db	39h			; 6.5 x 66
	db	3Ah			; 7.0 x 66
	db	3Bh			; 7.5 x 66
	db	3Ch			; 8.0 x 66
	db	75h			; 4.5 x 100
	db	76h			; 5.0 x 100
	db	77h			; 5.5 x 100
	db	78h			; 6.0 x 100
	db	79h			; 6.5 x 100
	db	7Ah			; 7.0 x 100
	db	7Bh			; 7.5 x 100

%ifdef FSB133_NATIVE
UnlockedCMOSTab:
	db	0			; Manual
	db	75h			; 4.5 x 100
	db	76h			; 5.0 x 100
	db	77h			; 5.5 x 100
	db	78h			; 6.0 x 100
	db	79h			; 6.5 x 100
	db	7Ah			; 7.0 x 100
	db	7Bh			; 7.5 x 100
	db	7Ch			; 8.0 x 100
	db	0F5h			; 4.5 x 133
	db	0F6h			; 5.0 x 133
	db	0F7h			; 5.5 x 133
	db	0F8h			; 6.0 x 133
	db	0F9h			; 6.5 x 133
	db	0FAh			; 7.0 x 133
	db	0FBh			; 7.5 x 133
%else					; FSB133_NATIVE
	UnlockedCMOSTab	equ	MendocinoUnlCMOSTab
%endif					; FSB133_NATIVE
%else					; PATCH_KLAMATH_MULT
%ifdef NEW_SPEED_ROUTINES
	UnlockedCMOSTab	equ	UnlockCMOSOrig
%endif					; NEW_SPEED_ROUTINES
%endif					; PATCH_KLAMATH_MULT

					; Prepare special offsets for the
					; patching process: For BIOSes that
					; don't necessarily need new speed
					; routines (i.e. BX boards without
					; the need for the Klamath patch),
					; only the relevant table offsets
					; can get patched instead of the
					; whole routines. If the new speed
					; routines are used, the following
					; offsets point to the original
					; tables, otherwise they point to
					; the new tables. The same is done
					; for the routines, only the other
					; way round.
					;
					; This way, the patch scripts can
					; simply perform both types of
					; patches without having to know
					; which options have been enabled in
					; the build, and at the same time
					; avoid doing useless modifications.
%ifdef NEW_SPEED_ROUTINES_OPTIONAL
%ifdef FSB133_SUPPORT
	SpdTabOffset	equ	SpeedTabOrig
	MultTabOffset	equ	MultTabOrig
%else					; FSB133_SUPPORT
	SpdTabOffset	equ	SpeedTab
	MultTabOffset	equ	MultiplierTab
%endif					; FSB133_SUPPORT
%endif					; NEW_SPEED_ROUTINES_OPTIONAL

%ifdef PATCH_SYMBIOS
%ifdef ONBOARD_SCSI
OnbSCSIOptions:
%else					; ONBOARD_SCSI
SymbiosOptions:
%endif					; ONBOARD_SCSI
	db	"Auto", 0
	db	"Disabled", 0
	db	"Enabled", 0
%endif					; PATCH_SYMBIOS

;----------------------------------------------------------------------------
; code
;----------------------------------------------------------------------------

%ifdef NEW_SPEED_ROUTINES

; =============== S U B R O U T I N E =======================================

fillSpeedDropdn:
	push	di
	call	isMultLockE000
	jb	short .end
	call	getMultIdxE000
%ifdef FSB133_SUPPORT
	mov	cx, 18h
%else					; FSB133_SUPPORT
	mov	cx, 10h
%endif					; FSB133_SUPPORT
	mul	cl
	lea	si, [SpeedTab]
%ifdef PATCH_KLAMATH_MULT
	pushad				; check for a Klamath core
	mov	eax, 1
	cpuid
	and	al, 0F0h
	cmp	ax, 630h
	popad
	jnz	.getOffset		; use the default table for any other
	lea	si, [KlamathSpeedTab]	; core, but use a patched up table
					; for Klamath (just for display)
%endif					; PATCH_KLAMATH_MULT

.getOffset:
	add	si, ax
	lea	di, [SpeedLockedEntr]
	cld

.copyLoop:
	lodsb
	mov	[di], al
	inc	di
	loop	.copyLoop

.end:
	pop	di
	retn
%else					; NEW_SPEED_ROUTINES
%ifdef NEW_SPEED_ROUTINES_OPTIONAL
	fillSpeedDropdn	equ	fillSpeedOrig
%endif					; NEW_SPEED_ROUTINES_OPTIONAL
%endif					; NEW_SPEED_ROUTINES


%ifdef NEW_SPEED_ROUTINES

; =============== S U B R O U T I N E =======================================


setMultTable:
	push	edi
%ifdef PATCH_KLAMATH_MULT
	push	eax
	push	ebx
	push	ecx
	push	edx
	mov	eax, 1
	cpuid
	and	al, 0F0h
	mov	di, ax
	pop	edx
	pop	ecx
	pop	ebx
	pop	eax
	cmp	di, 630h		; check for Klamath core
	jnz	.checkLock		; use a patched up table for Klamath
	pusha				; cores (regardless of a multiplier
	push	beforeUpdateTab		; lock)
	call	0F000h:dispatchToF000
	mov	ax, KlamathMultTab
	mov	word [fs:multTabOfs], ax
	push	afterUpdateTab
	call	0F000h:dispatchToF000
	popa
%endif					; PATCH_KLAMATH_MULT

.checkLock:
	call	isMultLockE000
	jb	.coreUnlocked		; jump if multiplier unlocked
	call	getMultIdxE000
	test	al, 10h
	jz	.setFSB			; jump if low multiplier
	pusha				; switch to high multiplier table
	push	beforeUpdateTab		; (only Coppermine and above cores
	call	0F000h:dispatchToF000	; have high multipliers, so we don't
	mov	ax, MultiplierTab	; need to differentiate the cores)
	mov	word [fs:multTabOfs], ax
	push	afterUpdateTab
	call	0F000h:dispatchToF000
	popa
	and	al, 0Fh

.setFSB:
%ifdef PLATFORM_APOLLO
	mov	ah, [bp+37h]
%elifdef PLATFORM_BX
	mov	ah, [bp+1Eh]
	shr	ah, 4
%endif					; PLATFORM_APOLLO / PLATFORM_BX
	cmp	dx, 1
	jb	short .setCMOSValue	; selection 0: "Manual", keep FSB
	mov	ah, 3			; selection 1: FSB 66
	jz	short .setCMOSValue
	mov	ah, 7			; selection 2: FSB 100
%ifdef FSB133_SUPPORT
	cmp	dx, 2
	jz	short .setCMOSValue
	mov	ah, 0Fh			; selection 3: FSB 133
%endif					; FSB133_SUPPORT

.setCMOSValue:
%ifdef PLATFORM_APOLLO
	mov	[bp+37h], ah
%elifdef PLATFORM_BX
	shl	ah, 4
	or	al, ah
%endif					; PLATFORM_APOLLO / PLATFORM_BX
	mov	[bp+1Eh], al
	jmp	short .end
; ---------------------------------------------------------------------------

.coreUnlocked:
	or	dx, dx
	jz	.end			; jump if "Manual" selected
	push	si			; else determine CMOS value from
	mov	si, dx			; selection
%ifdef PATCH_KLAMATH_MULT
	mov	al, [cs:KlamathUnlCMOSTab+si]
	cmp	di, 630h
	jz	.setCMOSValueFromTab
	mov	al, [cs:DeschutesUnlCMOSTab+si]
	cmp	di, 650h
	jz	.setCMOSValueFromTab
	mov	al, [cs:MendocinoUnlCMOSTab+si]
	cmp	di, 660h
	jz	.setCMOSValueFromTab
%endif					; PATCH_KLAMATH_MULT
	mov	al, [cs:UnlockedCMOSTab+si]

.setCMOSValueFromTab:
%ifdef PLATFORM_APOLLO
	push	ax
	and	al, 0Fh
	mov	[bp+1Eh], al
	pop	ax
	and	al, 0F0h
	shr	al, 4
	mov	[bp+37h], al
%elifdef PLATFORM_BX
	mov	[bp+1Eh], al
%endif					; PLATFORM_APOLLO / PLATFORM_BX
	pop	si

.end:
	pop	edi
	retn

%else					; NEW_SPEED_ROUTINES
%ifdef NEW_SPEED_ROUTINES_OPTIONAL
	setMultTable	equ	setMultTabOrig
%endif					; NEW_SPEED_ROUTINES_OPTIONAL
%endif					; NEW_SPEED_ROUTINES


%ifdef HARDWARE_MONITOR

; =============== S U B R O U T I N E =======================================


showCPUTemp:
	push	eax
%ifdef SMP
	push	ds
	mov	ax, 0E000h
	mov	ds, ax
	mov	si, cpuInfoList0
	cmp	ch, 1
	jz	short .checkType
	mov	si, cpuInfoList1

.checkType:
	mov	ax, [si+17h]
	pop	ds
%else					; SMP
	mov	al, byte [bp+cpuType]
%endif					; SMP
	and	al, 0F0h
	cmp	al, 40h
	pop	eax
	jnz	short .checkReading

	mov	cx, 1			; disable CPU temperature display for
	mov	si, di			; any Via CPU
	call	showDisable
	mov	si, aNA			; "N/A"
	call	printStringExt
	retn
; ---------------------------------------------------------------------------

.checkReading:
	call	unknown26
	push	bx
%if FEATURE_LEVEL >= 2
	push	unknown27
	call	0F000h:dispatchToF000
	mov	bx, ax
	pushf
	cmp	bx, 8000h
%else					; FEATURE_LEVEL
	call	unknown27
	pushf
	cmp	bx, 0FFFFh
%endif					; FEATURE_LEVEL
	jnz	short .showReading
	popf
	mov	cx, 1
	mov	si, di
	call	showDisable
	mov	si, aNA			; "N/A"
	call	printStringExt
	jmp	short .end
; ---------------------------------------------------------------------------

.showReading:
	popf
	call	unknown28
	call	unknown29
	mov	cx, 1
	mov	si, di
	mov	bx, [es:di]
	call	showEnable

.end:
	pop	bx
	retn


%ifdef PATCH_MINUS_5V_RAIL

; =============== S U B R O U T I N E =======================================


showVoltage:
	call	unknown26
	push	bx
%if FEATURE_LEVEL >= 2
	push	isVoltageOK		; check whether voltage is in limits
	call	0F000h:dispatchToF000
	call	unknown28
	push	readAndConvertV		; get current converted reading in mV
	call	0F000h:dispatchToF000
	cmp	ch, minus5VRailID	; patch for -5 V: If the converted
	jnz	short .print		; reading is at the end of the
	cmp	ax, -6140		; possible range, we assume that no
	jg	.print			; -5 V rail is present and disable
	mov	cx, 1			; the menu item
	mov	si, di
	call	showDisable
	mov	si, aNA			; "N/A"
	call	printStringExt
	jmp	short .end
;----------------------------------------------------------------------------

%else					; FEATURE_LEVEL
	mov	al, 20h
	add	al, ch
	push	readHWMonF000		; read requested voltage
	call	0F000h:dispatchToF000
	cmp	ch, minus5VRailID	; patch for -5 V: If the reading is
	jnz	short .acceptReading	; the maximum value, we assume that
	cmp	al, 0FFh		; no -5 V rail is present and disable
	jb	.acceptReading		; the menu item
	mov	cx, 1
	mov	si, di
	call	showDisable
	mov	si, aNA			; "N/A"
	call	printStringExt
	jmp	short .end
;----------------------------------------------------------------------------

.acceptReading:
	mov	bl, ch			; get the voltage range matching the
	push	getVoltageRange		; rail
	call	0E000h:dispatchToE000
	cmp	bh, al
	jb	short .rangeCompared
	cmp	al, bl

.rangeCompared:
	push	ax
	call	unknown28
	pop	ax
	push	convertVoltage		; convert raw reading to printable
	call	0E000h:dispatchToE000	; number
%endif					; FEATURE_LEVEL

.print:
	or	ax, ax
	jns	short .checkLeadingDigits
	dec	byte [bp+82h]		; print minus sign for negative
	push	ax			; numbers
	mov	al, '-'
	call	printCharExt
	pop	ax
	neg	ax

.checkLeadingDigits:
	xor	dx, dx			; value is in mV: convert to V
	mov	cx, 1000
	div	cx
	cmp	al, 10
	jnb	short .printVoltage
	push	ax			; print leading space for one-digit
	mov	al, ' '			; values
	call	printCharExt
	pop	ax

.printVoltage:
	push	dx			; print integer part
	xor	edx, edx
	mov	dx, ax
	call	printNumberExt
	mov	al, '.'			; print dot
	call	printCharExt
	pop	ax			; print first fractional digit
	mov	cl, 100
	div	cl
	push	ax
	xor	edx, edx
	mov	dl, al
	call	printNumberExt
	pop	ax			; print second fractional digit
	xor	al, al
	xchg	al, ah
	mov	cl, 10
	div	cl
	xor	edx, edx
	mov	dl, al
	call	printNumberExt
	mov	al, 'V'			; print unit
	call	printCharExt
	mov	cx, 1			; enable menu item
	mov	si, di
	call	showEnable

.end:
	pop	bx
	clc
	retn
%else					; PATCH_MINUS_5V_RAIL
	showVoltage	equ	showVoltageOrig
%endif					; PATCH_MINUS_5V_RAIL
%endif					; HARDWARE_MONITOR
