;****************************************************************************
; BIOS patch code for
; - ASUS CUBX 1008 beta 004 (1008cu.004)
; - ASUS CUBX-L/E 1008 beta 004 (1008cul.004)
; - ASUS CUV4X 1010 beta 003 (1010.003)
; - ASUS CUV4X-CME 1002 beta 002 (1002.002)
; - ASUS CUV4X-D 1016 (CUVD1016.AWD)
; - ASUS CUV4X-DLS 1016 (VDLS1016.AWD)
; - ASUS CUV4X-E (onboard audio) 1006 beta 005 (1006.005)
; - ASUS CUV4X-E (no onboard audio) 1005 beta 005 (1005.005)
; - ASUS CUV4X-M 1008 beta 003 (1008.003)
; - ASUS P3B-F 1008 beta 004 (1008f.004)
; - ASUS P3C-E (no audio/onboard audio) 1025 beta 002 (1025e.002)
; - ASUS P3C-E (software AC97) 1025 beta 002 (1025e_swa.002)
; - ASUS P3V133 1003 beta 002 (1003.002)
; - ASUS P3V4X 1006 beta 004 (1006.004)
; - ASUS TUV4X 1006 beta 002 (1006.002)
;****************************************************************************
;
; Boot block file containing new code and data to be added to the BIOS
;
;****************************************************************************
;
; Changes:
; - workarounds for Pentium Pro errata 5 (disable fast string operations for
;   CPUID 611h), 58 and 59 (disable streaming buffers for CPUID 619h)
;
;****************************************************************************

;----------------------------------------------------------------------------
; assembler directives/options
;----------------------------------------------------------------------------

[map symbols bin\boot.map]
bits 16
cpu 686

;----------------------------------------------------------------------------
; includes
;----------------------------------------------------------------------------

%include	"macro.def"		; macro definitions
%include	"bios.equ"		; addresses of BIOS code and data
					; used in patch routines

;----------------------------------------------------------------------------
; origin
;----------------------------------------------------------------------------

patchStart:
	org 	patchLocBoot		; go to free location in F000 segment
					; to place patch data and code into

;----------------------------------------------------------------------------
; data
;----------------------------------------------------------------------------

;----------------------------------------------------------------------------
; code
;----------------------------------------------------------------------------

%ifdef PATCH_PENTIUM_PRO

; =============== F U N C T I O N   C H U N K ===============================


initPentiumPro:
	mov	eax, 1
	cpuid
	cmp	ax, 611h		; disable fast string operations for
	jnz	short .disStrBuf	; Pentium Pro stepping B0 (CPUID
	mov	ecx, 1E0h		; 611h) -- erratum 5
	rdmsr
	and	eax, 2
	or	al, 4
	wrmsr
	jmp	short .done
; ---------------------------------------------------------------------------

.disStrBuf:
	cmp	ax, 619h		; disable streaming buffers for
	jnz	short .done		; Pentium Pro stepping sB1 (CPUID
	mov	ecx, 33h		; 619h) -- errata 58 and 59
	rdmsr
	or	eax, 40000000h
	wrmsr

.done:
	jmp	continuePOST

%endif					; PATCH_PENTIUM_PRO
