;****************************************************************************
; BIOS patch code for
; - ASUS CUBX 1008 beta 004 (1008cu.004)
; - ASUS CUBX-L/E 1008 beta 004 (1008cul.004)
; - ASUS CUV4X 1010 beta 003 (1010.003)
; - ASUS CUV4X-CME 1002 beta 002 (1002.002)
; - ASUS CUV4X-D 1016 (CUVD1016.AWD)
; - ASUS CUV4X-DLS 1016 (VDLS1016.AWD)
; - ASUS CUV4X-E (onboard audio) 1006 beta 005 (1006.005)
; - ASUS CUV4X-E (no onboard audio) 1005 beta 005 (1005.005)
; - ASUS CUV4X-M 1008 beta 003 (1008.003)
; - ASUS P3B-F 1008 beta 004 (1008f.004)
; - ASUS P3C-E (no audio/onboard audio) 1025 beta 002 (1025e.002)
; - ASUS P3C-E (software AC97) 1025 beta 002 (1025e_swa.002)
; - ASUS P3V133 1003 beta 002 (1003.002)
; - ASUS P3V4X 1006 beta 004 (1006.004)
; - ASUS TUV4X 1006 beta 002 (1006.002)
;****************************************************************************
;
; Main file containing new code and data to be added to the BIOS
;
;****************************************************************************
;
; Changes:
; - support of multipliers 12.5x through 16.0x on the "Advanced" tab of the
;   BIOS Setup Utility, including display of suggested CPU speeds for
;   standard FSB speeds
; - support of displaying normalized frequencies at bootup for multipliers
;   12.5x through 16.0x at standard FSB speeds
; - fix for the display of processor brand names at bootup, including support
;   for Pentium III-S and Pentium Pro processors
; - fix for the display of frequency suffixes (A, E, B, EB) at bootup
;   according to Intel datasheets
; - support of additional voltage ranges for processors with Tualatin core
;   and Pentium Pro processors to suppress Hardware Monitor errors at bootup
; - matched selection of multipliers for unlocked processors to core types
; - unlocked core voltages for undervolting
; - added Setup Utility "Enable" option for Symbios SCSI BIOS to force
;   loading the ROM module even if no matching adapter is present
; - POST fixes for Pentium Pro processors
; - fix for the "Processor Version" and "Processor Family" fields in SMBIOS
; - experimental support for Via C3 processors
; - fixed Hardware Monitor error for modern PSUs without -5 V rail
;
;****************************************************************************

;----------------------------------------------------------------------------
; assembler directives/options
;----------------------------------------------------------------------------

[map symbols bin\mainE000.map]
bits 16
cpu 686

;----------------------------------------------------------------------------
; includes
;----------------------------------------------------------------------------

%include	"macro.def"		; macro definitions
%include	"bios.equ"		; addresses of BIOS code and data
					; used in patch routines
%include	"awardext.equ"		; addresses of EXTGROUP patch code
					; and data

;----------------------------------------------------------------------------
; origin
;----------------------------------------------------------------------------

patchStart:
	org 	patchLocE000		; go to free location in E000 segment
					; to place patch data and code into

;----------------------------------------------------------------------------
; data
;----------------------------------------------------------------------------

%if FEATURE_LEVEL = 0
ManufacturerIDs:
	dw	IntelTypeMap		; Intel
	db	'Genu'
	db	'ntel'
	db	'ineI'
	dw	ViaTypeMap		; VIA
	db	'Cent'
	db	'auls'
	db	'aurH'
	dw	0FFFFh			; terminator
%endif					; FEATURE_LEVEL

IntelTypeMap:				; mapping of Intel processor CPUIDs
					; to BIOS-internal "types"
	dw	0630h, 02h		; Klamath core
	dw	0650h, 03h		; Deschutes core
					; (Covington core with same CPUID is
					; labeled as "type 1" via code)
	dw	0660h, 04h		; Mendocino core
	dw	0670h, 05h		; Katmai core
	dw	0680h, 06h		; Coppermine core
	dw	0690h, 07h		; Banias core
					; (not relevant, since only mobile
					; processors with this core exist)
	dw	06B0h, 08h		; Tualatin core
%ifdef PATCH_PENTIUM_PRO
	dw	0610h, 09h		; Pentium Pro
	dw	1630h, 0Ah		; Pentium II Overdrive (uses a
					; Deschutes 065x core)
%endif					; PATCH_PENTIUM_PRO
	dw	0FFFFh			; terminator

ViaTypeMap:
	dw	0660h, 40h		; mapping of Via processor CPUIDs to
					; BIOS-internal "types"
	dw	0670h, 41h		; cores from 0678h onwards are mapped
					; to "type 42h" via code
	dw	0680h, 43h
	dw	0690h, 44h
	dw	0FFFFh			; terminator

aPentium3S:
	db	'Intel(R) Pentium(R) III-S', 0
aSMBPentium3S:
	db	'Intel Pentium III-S Processor', 0
%ifdef PATCH_PENTIUM_PRO
aPentiumPro:
	db	'Intel(R) Pentium(R) Pro', 0
aSMBPentiumPro:
	db	'Intel Pentium Pro Processor', 0
aPentiumOverDrive:
	db	'Intel(R) Pentium(R) II OverDrive(R)', 0
aSMBPentiumOverDr:
	db	'Intel Pentium II OverDrive Processor', 0
%endif					; PATCH_PENTIUM_PRO
%if FEATURE_LEVEL < 4
aViaC3:
	db	'VIA C3', 0
aSMBViaC3:
	db	'VIA C3 Processor', 0
%endif					; FEATURE_LEVEL

%if FEATURE_LEVEL = 0
CPUSpeedLookup:				; selector table for CPU speed table
	dw	116			; according to FSB
	dw	CPUSpeedFSB133
	dw	83
	dw	CPUSpeedFSB100
	dw	0
	dw	CPUSpeedFSB66
%endif					; FEATURE_LEVEL

%if FEATURE_LEVEL < 4
CPUSpeedFSB133:
	dw	2133, 2066, 2000, 1933	; display values for processor
	dw	1866, 1800, 1733, 1666	; frequencies at 133 MHz FSB
	dw	1600, 1533, 1466, 1400
	dw	1333, 1266, 1200, 1133
	dw	1066, 1000, 933, 866
	dw	800, 733, 667, 600
	dw	533, 466, 400, 333
	dw	266
	dw	0			; terminator

CPUSpeedFSB100:
	dw	1600, 1550, 1500, 1450	; display values for processor
	dw	1400, 1350, 1300, 1250	; frequencies at 100 MHz FSB
	dw	1200, 1150, 1100, 1050
	dw	1000, 950, 900, 850
	dw	800, 750, 700, 650
	dw	600, 550, 500, 450
	dw	400, 350, 300, 250
	dw	200
	dw	0			; terminator

CPUSpeedFSB66:
	dw	1066, 1033, 1000, 966	; display values for processor
	dw	933, 900, 866, 833	; frequencies at 66 MHz FSB
	dw	800, 766, 733, 700
	dw	667, 633, 600, 566
	dw	533, 500, 466, 433
	dw	400, 366, 333, 300
	dw	266, 233, 200, 166
	dw	133
	dw	0
%endif					; FEATURE_LEVEL

%ifdef JUMPERFREE
%if FEATURE_LEVEL < 4
IntelMultIdxTab:
	db	06h, 02h, 04h, 00h	; index translation table for Intel
	db	07h, 03h, 05h, 01h	; P6 processor multipliers matching
	db	0Fh, 0Ah, 0Ch, 08h	; the bit fields in MSR 2Ah
	db	00h, 0Bh, 0Dh, 09h	;
	db	11h, 15h, 17h, 1Fh	; (indexes refer to the display
	db	12h, 16h, 10h, 1Fh	; table within EXTGROUP ROM)
	db	1Bh, 1Fh, 18h, 13h
	db	19h, 1Fh, 1Ah, 14h

ViaMultIdxTab1:
	db	06h, 02h, 04h, 13h	; index translation table for Via
	db	07h, 03h, 05h, 15h	; processor multipliers, valid
	db	11h, 0Ah, 0Ch, 08h	; for CPUIDs 066xh and 067xh
	db	17h, 0Bh, 18h, 09h

ViaMultIdxTab2:
	db	06h, 02h, 04h, 13h	; index translation table for Via
	db	07h, 03h, 05h, 12h	; processor multipliers, valid
	db	11h, 0Ah, 0Ch, 08h	; for CPUID 068xh
	db	17h, 0Bh, 10h, 09h
	db	00h, 15h, 17h, 00h
	db	1Dh, 16h, 1Ch, 14h
	db	18h, 1Ah, 1Bh, 19h
	db	00h, 1Fh, 00h, 1Eh
%endif					; FEATURE_LEVEL

ViaMultIdxTab3:
	db	06h, 1Bh, 00h, 13h	; index translation table for Via
	db	07h, 00h, 00h, 12h	; processor multipliers, valid
	db	11h, 0Ah, 0Ch, 08h	; for CPUID 069xh
	db	17h, 0Bh, 10h, 09h
	db	11h, 15h, 17h, 13h
	db	1Dh, 16h, 1Ch, 14h
	db	18h, 1Ah, 1Bh, 19h
	db	17h, 1Fh, 00h, 1Eh

%ifdef PATCH_VCORE_MENU
VoltMenuMin:
	db	00h			; 3.50V - allow from 3.50V down
	db	00h			; 3.40V - allow from 3.50V down
	db	00h			; 3.30V - allow from 3.50V down
	db	00h			; 3.20V - allow from 3.50V down
	db	00h			; 3.10V - allow from 3.50V down
	db	00h			; 3.00V - allow from 3.50V down
	db	01h			; 2.90V - allow from 3.40V down
	db	02h			; 2.80V - allow from 3.30V down
	db	03h			; 2.70V - allow from 3.20V down
	db	04h			; 2.60V - allow from 3.10V down
	db	05h			; 2.50V - allow from 3.00V down
	db	07h			; 2.40V - allow from 2.80V down
	db	08h			; 2.30V - allow from 2.70V down
	db	09h			; 2.20V - allow from 2.60V down
	db	0Ah			; 2.10V - allow from 2.50V down
	db	0FFh			; invalid setting
	db	0Bh			; 2.05V - allow from 2.40V down
	db	0Bh			; 2.00V - allow from 2.40V down
	db	0Ch			; 1.95V - allow from 2.30V down
	db	0Dh			; 1.90V - allow from 2.20V down
	db	0Dh			; 1.85V - allow from 2.20V down
	db	0Eh			; 1.80V - allow from 2.10V down
	db	10h			; 1.75V - allow from 2.05V down
	db	11h			; 1.70V - allow from 2.00V down
	db	12h			; 1.65V - allow from 1.95V down
	db	13h			; 1.60V - allow from 1.90V down
	db	14h			; 1.55V - allow from 1.85V down
	db	15h			; 1.50V - allow from 1.80V down
					; (will be patched down to 1.70V,
					; i.e. 17h, for modern cores)
	db	17h			; 1.45V - allow from 1.70V down
	db	18h			; 1.40V - allow from 1.65V down
	db	19h			; 1.35V - allow from 1.60V down
	db	1Ah			; 1.30V - allow from 1.55V down
%endif					; PATCH_VCORE_MENU
%endif					; JUMPERFREE

;----------------------------------------------------------------------------
; code
;----------------------------------------------------------------------------

; =============== S U B R O U T I N E =======================================


getCPUBrand0:
	mov	eax, 1			; second parameter: brand index
	cpuid
	push	bx
	movzx	ax, byte [bp+cpuType]	; first parameter: CPU type
	push	ax

	call	getCPUBrand
	add	esp, 4			; clean up parameters

	retn


%if FEATURE_LEVEL >= 3

; =============== S U B R O U T I N E =======================================


getCPUBrand1:
	movzx	ax, byte [cs:di+19h]	; second parameter: brand index
	push	ax
	mov	ax, word [cs:di+17h]	; first parameter: CPU type
	push	ax

	call	getCPUBrand
	add	esp, 4			; clean up parameters

	retn

%endif					; FEATURE_LEVEL


; =============== S U B R O U T I N E =======================================


getCPUBrand:
	.type	equ 2
	.brand	equ 4

	mov	ax, cs
	mov	ds, ax
					; Here, we deduce the brand string
					; of the processor according to the
					; coarse typing done in
					; determineCPUType. We try to get the
					; "easy" ones out of the way first
					; and do the more complicated ones
					; (the various flavors of
					; Pentium IIIs and Pentium III-based
					; Celerons) at the end.

	mov	si, aViaC3		; "VIA C3"
	test	byte [esp+.type], 40h
	jnz	short .end

	mov	si, aCeleron		; "Intel(R) Celeron(TM)"
	cmp	byte [esp+.type], 1	; "Type 1" are Intel processors
	jz	short .end		; without L2 cache (Covington core).
					; These are always Celerons.

	cmp	byte [esp+.type], 4	; Mendocino cores are always Celerons
	jz	short .end

%ifdef PATCH_PENTIUM_PRO
	mov	si, aPentiumPro		; "Intel(R) Pentium(R) Pro"
	cmp	byte [esp+.type], 9
	jz	short .end

	mov	si, aPentiumOverDrive	; "Intel(R) Pentium(R) II OverDri..."
	cmp	byte [esp+.type], 0Ah
	jz	short .end
%endif					; PATCH_PENTIUM_PRO

	mov	si, aPentium3		; "Intel(R) Pentium(R) III"
	cmp	byte [esp+.type], 5	; Katmai cores are always Pentium
	jz	short .end		; IIIs

	cmp	byte [esp+.type], 8	; Tualatin cores might be Celerons,
	jz	short .checkP3S		; Pentium IIIs or Pentium III-Ss. We
					; check the latter first.

	cmp	byte [esp+.type], 6	; Coppermine cores might be Celerons
	jz	short .checkP3Celeron	; or Pentium IIIs.

	jmp	short .default
; ---------------------------------------------------------------------------

.checkP3S:
	call	getL2CacheSize		; Tualatin core: Check the cache size
	cmp	al, 6			; (in 8 KB units: 2^6 * 8 KB = 512
	jnz	short .checkP3Celeron	; KB). A Tualatin core with 512 KB of
					; L2 cache is always a Pentium III-S,
					; otherwise it is a normal Pentium
					; III or a Celeron.
	mov	si, aPentium3S		; "Intel(R) Pentium(R) III-S"
	jmp	short .end
; ---------------------------------------------------------------------------

.checkP3Celeron:
	test	byte [esp+.brand], 1	; These families support the CPUID
					; brand index feature, so we try to
					; identify them as Celerons.
	jz	short .end
	mov	si, aCeleron		; "Intel(R) Celeron(TM)"
	jmp	short .end
; ---------------------------------------------------------------------------

.default:
	mov	si, aPentium2		; "Intel(R) Pentium(R) II"
					; If no other type could be
					; identified, we are looking at a
					; regular Klamath or Deschutes core,
					; i.e. a Pentium II.

.end:
	retn


; =============== S U B R O U T I N E =======================================


printCPUSpeed0:
	mov	ax, word [bp+cpuFreq]	; third parameter: CPU frequency
	push	ax
	mov	eax, 1			; second parameter: brand index
	cpuid
	push	bx
	movzx	ax, byte [bp+cpuType]	; first parameter: CPU type
	push	ax

	call	printCPUSpeed
	add	esp, 6			; clean up parameters

	retn


%if FEATURE_LEVEL >= 3

; =============== S U B R O U T I N E =======================================


printCPUSpeed1:
	mov	ax, word [di+15h]	; third parameter: CPU frequency
	push	ax
	movzx	ax, byte [di+19h]	; second parameter: brand index
	push	ax
	mov	ax, word [di+17h]	; first parameter: CPU type
	push	ax

	call	printCPUSpeed
	add	esp, 6			; clean up parameters

	retn

%endif					; FEATURE_LEVEL


; =============== S U B R O U T I N E =======================================


printCPUSpeed:
	.type	equ 2
	.brand	equ 4
	.freq	equ 6

	inc	byte [bp+screenCol]	; go to next column on screen
	mov	ax, [esp+.freq]		; print the raw display frequency
	call	printNumber

	cmp	byte [esp+.type], 8	; check for various core types
	jnz	short .checkCoppermine	; and call special handling routines
	call	printTualatin		; for each one
	jmp	short .printUnit
; ---------------------------------------------------------------------------

.checkCoppermine:
	cmp	byte [esp+.type], 6
	jnz	short .checkKatmai
	call	printCoppermine
	jmp	short .printUnit
; ---------------------------------------------------------------------------

.checkKatmai:
	cmp	byte [esp+.type], 5
	jnz	short .checkMendocino
	call	printKatmai
	jmp	short .printUnit
; ---------------------------------------------------------------------------

.checkMendocino:
	cmp	byte [esp+.type], 4
	jnz	short .checkVia
	call	printMendocino
	jmp	short .printUnit
; ---------------------------------------------------------------------------

.checkVia:
	test	byte [esp+.type], 40h
	jz	short .printUnit
	call	printVia

.printUnit:
	mov	si, aMHz		; " MHz"
	call	printString		; finish by printing the "MHz" suffix
	retn


; =============== S U B R O U T I N E =======================================


printTualatin:
	.type	equ 2
	.brand	equ 4
	.freq	equ 6

	call	getL2CacheSize		; Pentium III-S CPUs need no special
	cmp	al, 6			; discriminator
	jz	short .end
	cmp	word [esp+.freq], 1133	; All other Tualatin cores between
	ja	short .end		; 1000 and 1133 MHz get an "A"
	cmp	word [esp+.freq], 1000	; suffix. (The Tualatin-based Celeron
	jb	short .end		; at 900 MHz should have a suffix,
					; but doesn't have one.)
	mov	si, aA			; "A"
	call	printString

.end:
	retn


; =============== S U B R O U T I N E =======================================


printCoppermine:
	.type	equ 2
	.brand	equ 4
	.freq	equ 6

	test	byte [esp+.brand], 1	; check for the Coppermine-based
	jz	short .checkPentium	; Celeron 533A
	cmp	word [esp+.freq], 533
	jnz	short .end
	mov	si, aA			; "A"
	call	printString
	jmp	short .end
; ---------------------------------------------------------------------------

.checkPentium:
	cmp	word [bp+cpuFSB], 100	; Coppermines at 100 MHz FSB between
	jnz	short .checkFSB133	; 500 and 600 MHz are distinguished
	cmp	word [esp+.freq], 600	; from their Katmai counterparts by
	ja	short .end		; an "E" suffix
	cmp	word [esp+.freq], 500
	jb	short .end
	mov	si, aE			; "E"
	call	printString
	jmp	short .end
; ---------------------------------------------------------------------------

.checkFSB133:
	cmp	word [bp+cpuFSB], 133	; Coppermines at 133 MHz FSB between
	jnz	short .end		; 533 and 600 MHz are suffixed with
	cmp	word [esp+.freq], 1000	; "EB". The 1000 MHz processor only
	jz	short .printB		; gets a "B" as it has no Katmai
	cmp	word [esp+.freq], 800	; counterpart. The latter is also
	jz	short .printE		; true for the 800 MHz processor, but
	cmp	word [esp+.freq], 533	; it gets the full suffix "EB"
	jb	short .end		; anyway.
	cmp	word [esp+.freq], 600
	ja	short .end

.printE:
	mov	si, aE			; "E"
	call	printString

.printB:
	mov	si, aB			; "B"
	call	printString

.end:
	retn


; =============== S U B R O U T I N E =======================================


printKatmai:
	.type	equ 2
	.brand	equ 4
	.freq	equ 6

	cmp	word [bp+cpuFSB], 133	; Katmai-based processors at 133 MHz
	jnz	short .end		; FSB between 533 and 600 MHz are
	cmp	word [esp+.freq], 533	; suffixed with "B" to distinguish
	jb	short .end		; them from their 100 MHz FSB
	cmp	word [esp+.freq], 600	; counterparts.
	ja	short .end
	mov	si, aB			; "B"
	call	printString

.end:
	retn


; =============== S U B R O U T I N E =======================================


printMendocino:
	.type	equ 2
	.brand	equ 4
	.freq	equ 6

	cmp	word [esp+.freq], 300	; check for the Mendocino-based
	jnz	short .end		; Celeron 300A
	mov	si, aA			; "A"
	call	printString

.end:
	retn


; =============== S U B R O U T I N E =======================================


printVia:
	.type	equ 2
	.brand	equ 4
	.freq	equ 6

	cmp	byte [esp+.type], 40h	; all C3 processors except Samuel
	jz	.end			; cores (CPUID 66xh) get an "A"
					; suffix
	mov	si, aA			; "A"
	call	printString

.end:
	retn


%ifdef JUMPERFREE

; =============== S U B R O U T I N E =======================================


getMultIdx:
	push	si
	push	edx
	push	ebx
	mov	bl, 2Fh			; See below: Via cores from (at
	call	checkType4x		; least) Nehemiah onwards use the
	jz	short .getMultiplier	; additional bit. In previous cores
					; the bit is reserved, but should
					; return 0 according to datasheets,
					; so using it anyway shouldn't
					; hurt(?).
	mov	eax, 1
	cpuid
	and	ax, 0FFFh
	mov	bl, 0Fh			; Prepare the bit mask for the
	cmp	ax, 683h		; multiplier bit field in MSR 2Ah:
	jb	short .checkTualatin	; CPUs from 0683h onwards have an
	cmp	ax, 68Fh		; additional bit for the multiplier.
	ja	short .checkTualatin	; (This does not apply to 069xh,
	mov	bl, 2Fh			; which are mobile procesors anyway.)

.checkTualatin:
	and	ax, 0FF0h		; Tualatin cores also use the
	cmp	ax, 6B0h		; additional bit.
	jnz	short .getMultiplier
	mov	bl, 2Fh

.getMultiplier:
	mov	ecx, 2Ah		; get the multiplier bit field from
	rdmsr				; MSR 2Ah
	shr	eax, 16h
	and	al, bl
	test	al, 20h			; stuff the bits together (the
	jz	short .codeToMultIdx	; additional bit is separated from
	and	al, 0DFh		; the rest)
	or	al, 10h			

.codeToMultIdx:
	xor	ah, ah
	call	getMultIdxTab		; get the multiplier index
	add	si, ax			; translation table matching the
	mov	al, [cs:si]		; processor type and find the index
	pop	ebx			; into the actual mutiplier table
	pop	edx
	pop	si
	retn


; =============== S U B R O U T I N E =======================================


getMultIdxTab:
	mov	si, IntelMultIdxTab	; assume Intel by default
	call	checkType4x
	jnz	short .end
	mov	si, ViaMultIdxTab2	; newer Via processors use extended
	call	checkType43		; multiplier tables, otherwise we
	jz	.end			; take the short one
	mov	si, ViaMultIdxTab3
	call	checkType44
	jz	.end
	mov	si, ViaMultIdxTab1

.end:
	retn

%endif					; JUMPERFREE


%ifdef PATCH_VOLTAGE_RANGE

; =============== S U B R O U T I N E =======================================


getCPUVoltageRange:
	mov	bx, 8970h		; Covington, Deschutes, Mendocino,
	call	checkType1		; Katmai: ~ 2.0 V +/- 0.2 V
	jz	short .end
	call	checkType3
	jz	short .end
	call	checkType4
	jz	short .end
	call	checkType5
	jz	short .end

	mov	bx, 735Ah		; Coppermine: ~ 1.65 V +/- 0.2 V
	call	checkType6
	jz	short .end

	mov	bx, 6550h		; Tualatin: ~ 1.45 V +/- 0.17 V
	call	checkType8
	jz	short .end

%ifdef PATCH_PENTIUM_PRO
	mov	bx, 0E6B6h		; Pentium Pro: ~ 3.3 V +/- 0.38 V
	call	checkType9		; (models vary from 3.1 V to 3.5 V)
	jz	short .end

	mov	bx, 0A88Fh		; OverDrive: ~ 2.5 V +/- 0.2 V
	call	checkTypeA		; (primary Vcc)
	jz	short .end
%endif					; PATCH_PENTIUM_PRO

	mov	bx, 836Ah		; C5A/Samuel: ~ 1.9 V +/- 0.2 V
	call	checkType40		; (models vary from 1.8 V to 2.0 V)
	jz	short .end

	mov	bx, 6F56h		; C5B/Samuel 2: ~ 1.575 V +/- 0.2 V
	call	checkType41		; (models vary from 1.5 V to 1.65 V)
	jz	short .end

	mov	bx, 5F4Ah		; C5C/Ezra, C5M/C5N/Ezra-T,
	call	checkType42		; C5XL/Nehemiah:~ 1.35 V +/- 0.17 V
	jz	short .end		; (Nehemiah models vary from 1.25 V
	call	checkType43		; to 1.45 V)
	jz	short .end
	call	checkType44
	jz	short .end

	mov	bx, 0BBA2h		; Klamath/default: ~ 2.8 V +/- 0.2 V

.end:
	retn

%endif					; PATCH_VOLTAGE_RANGE


%ifdef JUMPERFREE
%ifdef PATCH_KLAMATH_MULT

; =============== S U B R O U T I N E =======================================


setCPUParams:
	call	unknown1
	jnb	.end
	push	es
	mov	ax, 0F000h
	mov	es, ax
	call	initMenuChange
	call	isMultLock
	jnb	short .checkSafeMode	; jump if multiplier locked
	push	edi
	push	eax
	push	ebx
	push	ecx
	push	edx
	mov	eax, 1
	cpuid
	and	al, 0F0h
	mov	di, ax
	pop	edx
	pop	ecx
	pop	ebx
	pop	eax
	mov	ax, KlamathUSpTab	; multiplier unlocked: choose
	cmp	di, 630h		; suitable speed table matching
	jz	.setSpeedTable		; processor core
	mov	ax, DeschutesUSpTab
	cmp	di, 650h
	jz	.setSpeedTable
%ifdef PATCH_PENTIUM_PRO
	cmp	di, 1630h		; OverDrive processors have Deschutes
	jz	.setSpeedTable		; cores
%endif					; PATCH_PENTIUM_PRO
	mov	ax, MendocinoUSpTab
	cmp	di, 660h
	jz	.setSpeedTable
	mov	ax, UnlockedSpdTab

.setSpeedTable:
	mov	[es:speedTabOfs], ax
	pop	edi

.checkSafeMode:
	mov	al, bootState
	call	getFromCMOS
	test	al, 30h
	jz	short .done
	or	byte [unknown3], 2
	call	hasSpeedFailed
	jb	short .setMsgSpeed
	call	isMultLock
	mov	ax, aNewCPUUnlock
	jb	short .setMsgNewCPU
	mov	ax, aNewCPULock

.setMsgNewCPU:
	mov	[es:speedHelpOfs], ax
	and	byte [bp+1Dh], 0F0h
	jmp	short .done
; ---------------------------------------------------------------------------

.setMsgSpeed:
	lea	si, [speedMenu]
	call	getSetupOption
	or	al, al
	jz	short .setMsgFrequency
	mov	ax, aImproperSpeed
	mov	[es:speedHelpOfs], ax
	jmp	short .done
; ---------------------------------------------------------------------------

.setMsgFrequency:
	mov	ax, aImproperFreq
	mov	[es:multHelpOfs], ax
	mov	[es:freqHelpOfs], ax

.done:
	call	endMenuChange
	pop	es

.end:
	retn

%endif					; PATCH_KLAMATH_MULT


%ifdef PATCH_VCORE_MENU

; =============== S U B R O U T I N E =======================================

getVCoreHighLimit:
	mov	al, [cs:VoltMenuMin+si]
	cmp	al, 15h			; apply patch for processors with a
	jnz	short .end		; default VID request of 1.50V: set
	call	checkType8		; the upper limit to 1.70V for
	jnz	short .end		; Tualatin cores and leave it at
	mov	al, 17h			; 1.80V for all other cores (e.g.
					; Coppermine Celerons)

.end:
	retn

%endif					; PATCH_VCORE_MENU


%ifdef PATCH_VERIFY_VCORE

; =============== S U B R O U T I N E =======================================


verifyVCore:
	call	hasSpeedFailed
	jnb	short .resetVCore
%ifdef PLATFORM_BX
	mov	al, bootState0
	call	getFromCMOS
	test	al, 80h
	jnz	short .resetVCore
%endif					; PLATFORM_BX
	mov	cx, 9085h
	call	unknown7
	test	al, 20h
	jnz	short .compareVCore
	call	unknown8
	jmp	short .verifyOK
; ---------------------------------------------------------------------------

.compareVCore:
	and	al, 1Fh
	mov	ah, al			; AH: VID set in setup menu
	mov	al, vidRequest
	call	getFromCMOS
	and	al, 1Fh			; AL: VID requested by CPU
	cmp	al, 1Fh
	jz	short .resetVCore
	mov	cl, al
	xor	ah, 10h
	xor	al, 10h
	movzx	si, al
	call	getVCoreHighLimit	; get first list entry (i.e. highest
					; VID allowed)
	cmp	ah, al
	jb	short .resetVCore	; reset if VID before first allowed
					; list entry (i.e. VID too large)
	mov	ch, 1Fh

.compareLast:
	cmp	ah, ch
	ja	.resetVCore		; reset if VID after last allowed
					; list entry
	mov	al, cl
	jmp	short .verifyOK
; ---------------------------------------------------------------------------

.resetVCore:
	call	unknown9
	mov	cx, 9085h
	xor	al, al
	call	unknown10
	mov	ecx, 100000h

.outloop:
	out	0E1h, al
a32	loop 	.outloop
	call	unknown11
	call	unknown8
	stc
	jmp	.end
; ---------------------------------------------------------------------------

.verifyOK:
	clc

.end:
	retn

%endif					; PATCH_VERIFY_VCORE


%ifdef PATCH_VCORE_MENU

; =============== S U B R O U T I N E =======================================


setupVCoreMenu:
	push	es
	call	initMenuChange
	call	verifyVCore
	pushf
	mov	bx, 0F000h
	mov	es, bx
	lea	bx, [voltMenu]
	xor	ah, ah
	xor	al, 10h
	popf
	jnb	short .modifyMenu
	and	byte [bp+vid], 0E0h
	or	[bp+vid], al

.modifyMenu:
	cmp	al, 0Fh			; don't show menu in case of an
	jz	short .end		; invalid VID request

	and	word [es:bx], 0FFEFh	; remove hidden flag
	mov	[es:bx+15h], ax		; set default value
	mov	si, ax
	call	getVCoreHighLimit
	mov	[es:bx+0Dh], ax		; set minimum value
	mov	si, 1Fh			; fill rest of list with as many
					; options as possible

.setMax:	
	mov	[es:bx+0Fh], si		; set maximum value
	mov	si, [es:bx+0Bh]
	mov	ah, 6
	mul	ah
	add	si, ax
	mov	[es:bx+0Bh], si		; set first menu item

.end:
	call	endMenuChange
	pop	es
	retn

%endif					; PATCH_VCORE_MENU
%endif					; JUMPERFREE


%ifdef PATCH_SYMBIOS

; =============== S U B R O U T I N E =======================================


loadPCIExt:
%ifdef ONBOARD_NIC
	mov	si, onboardNICMenu
	call	getSetupOption
	or	al, al
	jz	short .afterNIC
	call	loadNICROM
%endif					; ONBOARD_NIC

.afterNIC:
%ifdef ONBOARD_ATA
	mov	si, onboardATAMenu
	call	getSetupOption
	or	al, al
	jz	short .afterATAFirst
	call	searchATA
%elifdef ONBOARD_SCSI
	mov	si, onbSCSIMenu
	call	getSetupOption
	test	al, 1
	jnz	short .afterSCSIFirst
%endif					; ONBOARD_ATA / ONBOARD_SCSI

.afterATAFirst:
%ifndef ONBOARD_SCSI
	mov	si, symbiosMenu		; get setup option for Symbios option
	call	getSetupOption		; ROM
	test	al, 1
	jnz	short .afterSCSIFirst	; if disabled, skip initialization
%else					; ONBOARD_SCSI
	push	ax			; save menu choice
	mov	si, onbSCSI1stMenu	; check whether onboard SCSI should
	call	getSetupOption		; have preference
	or	al, al
	pop	ax			; restore menu choice
	jz	short .lookForSymbios
	pushad				; if so, load the onboard ROM
	mov	ebx, 40040000h		; now
	call	uncompress
	popad
	call	searchOnbSCSI		; search for SCSI host adapter
%endif					; ONBOARD_SCSI

.lookForSymbios:
	call	searchSymbios		; search for Symbios adapter and
					; load ROM

.afterSCSIFirst:
	mov	ecx, 80000000h
	call	unknown12
	mov	ecx, 80000000h
	call	unknown13

%ifdef ONBOARD_ATA
	mov	si, onboardATAMenu
	call	getSetupOption
	or	al, al
	jnz	short .pciSnoop
	call	searchATA
%elifdef ONBOARD_SCSI
	mov	si, onbSCSIMenu
	call	getSetupOption
	test	al, 1
	jnz	short .pciSnoop
	mov	si, onbSCSI1stMenu
	call	getSetupOption
	or	al, al
	jnz	short .pciSnoop
	pushad
	mov	ebx, 40040000h		; load the onboard ROM
	call	uncompress
	popad
	call	searchOnbSCSI
%endif					; ONBOARD_ATA / ONBOARD_SCSI

.pciSnoop:
	mov	si, pciSnoopMenu
	call	getSetupOption
	or	al, al
	jz	short .end
	mov	ecx, 800000h
	mov	cx, [bp+0E4h]
	cmp	cx, 0
	jz	short .end
	shl	ecx, 8
	mov	cl, 4
	call	queryPCI
	test	al, 3
	jz	short .end
	or	al, 20h
	call	unknown14

.end:
	retn


; =============== S U B R O U T I N E =======================================


searchSymbios:
	pushad
	cmp	al, 2			; if setup option is "Enabled", force
	jz	.found			; load of option ROM, otherwise start
					; searching for a matching adapter
%ifdef PLATFORM_820
	mov	ecx, 80020000h
%else					; PLATFORM_820
	mov	ecx, 80000000h
%endif					; PLATFORM_820

.continue:
	xor	cl, cl
	call	queryPCI		; query PCI configuration space for
	cmp	ax, 1000h		; vendor 1000h: Symbios(/LSI/NCR)
	jnz	short .toNextBlock
	mov	cl, 2
	call	queryPCI		; query PCI configuration space for
	cmp	ax, 5			; device id 0001h, 0005h or 0006h
	jz	short .found
	cmp	ax, 6
	jz	short .found
	cmp	ax, 1
	jnz	short .toNextBlock

.found:
	mov	si, symbiosModuleID
	call	loadROMModule		; search and unpack ROM module
	jb	short .end
	call	unknown15
	jmp	short .end
; ---------------------------------------------------------------------------

.toNextBlock:
	add	ch, 8
	jnb	short .continue

.end:
	popad
	retn

%endif					; PATCH_SYMBIOS


; =============== S U B R O U T I N E =======================================


determineCPUType0:
	call	determineCPUType1
	call	storeCPUType

%ifndef SMP
	mov	ecx, 1Bh		; APIC global disable
	rdmsr
	and	ah, 0F7h
	wrmsr
%endif					; SMP

%if FEATURE_LEVEL = 4
	push	ds
	push	cs
	pop	ds
	call	storeExtCPUInfo
	pop	ds
%endif					; FEATURE_LEVEL
	retn


; =============== S U B R O U T I N E =======================================


determineCPUType1:
	push	ds
	push	si
	push	di
	push	cs
	pop	ds
	xor	eax, eax		; get CPU manufacturer
	cpuid
	mov	si, ManufacturerIDs

.checkManufacturer:
	cmp	word [si], 0FFFFh	; end of list reached?
	jz	short .useFallback
	cmp	ebx, [si+2]		; compare CPUID manufacturer ID to
	jnz	short .nextManufacturer	; list value
	cmp	ecx, [si+6]
	jnz	short .nextManufacturer
	cmp	edx, [si+0Ah]
	jz	short .checkModel	; manufacturer found, check CPU model

.nextManufacturer:
	add	si, 0Eh			; no match, go to next manufacturer
	jmp	short .checkManufacturer
;----------------------------------------------------------------------------

.checkModel:
	mov	si, [si]		; switch to model list
	mov	di, si
	mov	eax, 1			; get CPU model
	cpuid
	mov	ebx, eax		; split into family/model ...
	and	al, 0F0h
	and	bx, 0Fh			; ... and stepping

.nextModel:
	cmp	word [si], 0FFFFh	; end of list reached?
	jz	short .assumeDefault
	cmp	ax, [si]		; compare CPUID model to list value
	jz	short .modelFound	; model found
	add	si, 4
	jmp	short .nextModel	; no match, go to next model
;----------------------------------------------------------------------------

.assumeDefault:
	mov	si, IntelTypeMap

.modelFound:
	cmp	di, IntelTypeMap	; Intel CPU identified?
	jnz	short .checkVia
	cmp	ax, 650h		; CPUID 650h might be either a
	jnz	short .acceptType	; Deschutes or Covington core. A
	mov	ecx, 11Eh		; check for the presence of L2 cache
	rdmsr				; determines which core has been
	mov	dx, 1			; found. In case of a Covington core,
	shr	eax, 17h		; type 1 is selected.
	and	al, 1
	cmp	al, 1
	jz	short .storeType

.checkVia:
	cmp	di, ViaTypeMap		; Via CPU identified?
	jnz	short .acceptType
	cmp	ax, 670h		; CPUIDs 0677h and below belong to a
	jnz	short .acceptType	; Samuel 2 core, 0678h and above to
	cmp	bl, 8			; an Ezra core
	jb	short .acceptType
	mov	dx, 42h
	jmp	short .storeType
;----------------------------------------------------------------------------

.acceptType:
	mov	dx, [si+2]		; copy type from model list
	jmp	short .storeType

.useFallback:
	mov	dx, 1

.storeType:
	mov	ax, dx

.end:
	pop	di
	pop	si
	pop	ds
	retn


%ifdef PATCH_PENTIUM_PRO

; =============== S U B R O U T I N E =======================================


configureCaches:
	or	al, al
	jnz	short .local1
	mov	eax, 1			; only perform MTRR control for cores
	cpuid				; below Klamath
	and	ax, 0FFFh
	cmp	ax, 630h
	jge	.mtrrCtrlDone
	xor	eax, eax
	mov	edx, eax
	mov	ecx, 259h
	wrmsr
	mov	cl, 68h

.loop1:
	wrmsr
	inc	cl
	cmp	cl, 6Fh
	jbe	short .loop1
	xor	cl, cl

.loop2:
	wrmsr
	inc	cl
	cmp	cl, 0Fh
	jbe	short .loop2

.mtrrCtrlDone:
	mov	eax, 6060606h
	mov	edx, eax
	mov	ecx, 250h
	wrmsr
	mov	cl, 58h
	wrmsr
	mov	eax, 0C00h
	xor	edx, edx
	mov	cl, 0FFh
	wrmsr
	push	bp
	call	configureL2
	pop	bp
	jmp	.local6
; ---------------------------------------------------------------------------

.local1:
	mov	ecx, 26Fh
	mov	eax, 5050505h
	mov	edx, 5050505h
	wrmsr
	dec	cl
	wrmsr
	push	ds
	mov	ax, 0C000h
	mov	ds, ax
	cmp	word [0], 0AA55h
	jnz	short .local2
	mov	eax, 5050505h
	mov	edx, eax
	mov	ecx, 268h
	wrmsr

.local2:
	pop	ds
	mov	eax, 1			; only perform check of L2 cache
	cpuid				; address range for cores from
	and	ax, 0FFFh		; Klamath onwards
	cmp	ax, 630h
	jb	short .l2CacheRangeDone
	mov	ecx, 11Eh
	rdmsr
	shr	eax, 20
	and	al, 7
	mov	edi, 7FC00h
	cmp	al, 0
	jz	short .local3
	mov	edi, 0FFC00h
	cmp	al, 1
	jz	short .local3
	mov	edi, 1FFC00h
	cmp	al, 2
	jz	short .local3
	mov	edi, 3FFC00h

.local3:
	cmp	esi, edi
	jbe	.l2CacheRangeDone
	mov	esi, edi

.l2CacheRangeDone:
	mov	eax, esi
	or	eax, eax
	jz	short .end
	add	eax, 400h
	shl	eax, 0Ah
	call	unknown16
	mov	ecx, esi
	mov	si, memHoleMenu
	call	getSetupOption
	mov	bl, al
	or	bl, bl
	jz	short .local6
	mov	ax, 0F0h
	cmp	bl, 1
	jz	short .local4
	mov	ah, 0E0h
	cmp	bl, 2
	jz	short .local4
	mov	ah, 0C0h

.local4:
	shl	eax, 10h
	xor	edx, edx
	wrmsr
	mov	eax, 0FFF0h
	cmp	bl, 1
	jz	short .local5
	mov	al, 0E0h
	cmp	bl, 2
	jz	short .local5
	mov	al, 0C0h

.local5:
	shl	eax, 10h
	mov	ax, 800h
	mov	edx, 0Fh
	inc	ecx
	wrmsr

.local6:
	wbinvd
	mov	eax, cr3
	mov	cr3, eax

.end:
	retn

%endif					; PATCH_PENTIUM_PRO

%if FEATURE_LEVEL < 4

; =============== S U B R O U T I N E =======================================


checkType8:
	cmp	byte [bp+cpuType], 8
	retn

%endif					; FEATURE_LEVEL

%ifdef PATCH_PENTIUM_PRO

; =============== S U B R O U T I N E =======================================


checkType9:
	cmp	byte [bp+cpuType], 9
	retn


; =============== S U B R O U T I N E =======================================


checkTypeA:
	cmp	byte [bp+cpuType], 0Ah
	retn

%endif					; PATCH_PENTIUM_PRO

%if FEATURE_LEVEL = 0

; =============== S U B R O U T I N E =======================================


checkType40:
	cmp	byte [bp+cpuType], 40h
	retn

%endif					; FEATURE_LEVEL

%if FEATURE_LEVEL < 4

; =============== S U B R O U T I N E =======================================


checkType41:
	cmp	byte [bp+cpuType], 41h
	retn


; =============== S U B R O U T I N E =======================================


checkType42:
	cmp	byte [bp+cpuType], 42h
	retn

%endif					; FEATURE_LEVEL


; =============== S U B R O U T I N E =======================================


checkType43:
	cmp	byte [bp+cpuType], 43h
	retn


; =============== S U B R O U T I N E =======================================


checkType44:
	cmp	byte [bp+cpuType], 44h
	retn


; =============== S U B R O U T I N E =======================================


checkType4x:
	push	eax
	mov	al, byte [bp+cpuType]
	and	al, 0F0h
	cmp	al, 40h
	pop	eax
	retn


; =============== S U B R O U T I N E =======================================


controlL2Cache:
%ifdef PATCH_PENTIUM_PRO
	push	ax
	mov	eax, 1
	cpuid
	cmp	al, 30h
	pop	ax
	jnb	short .checkVia
	mov	bl, al			; perform L2 cache control for
	mov	ecx, 119h		; Pentium Pro cores
	rdmsr
	and	eax, 0FFF7FFFFh
	cmp	bl, 0FFh
	jz	short .configure
	or	eax, 80000h
	jmp	short .configure
; ---------------------------------------------------------------------------
%endif					; PATCH_PENTIUM_PRO

.checkVia:
	call	checkType4x		; only perform L2 cache control for
	jnz	short .end		; Via cores from CPUID 67xh onwards
	call	checkType40
	jz	short .end
	mov	bl, al
	mov	ecx, 1107h
	rdmsr
	and	eax, 0FFFFFEFFh
	cmp	bl, 0FFh
	jz	short .configure
	or	eax, 100h

.configure:
	wrmsr

.end:
	retn


; =============== S U B R O U T I N E =======================================


getL2CacheSize:
	push	ebx
	push	ecx
	push	edx
	call	checkType4x		; Via CPU?
	jz	short .via
	mov	eax, 2			; no, get cache size "Intel style"
	cpuid
	call	getCacheDesc
	jnb	short .end
	mov	eax, ebx
	call	getCacheDesc
	jnb	short .end
	mov	eax, ecx
	call	getCacheDesc
	jnb	short .end
	mov	eax, edx
	call	getCacheDesc
	jnb	short .end
	jmp	short .none
; ---------------------------------------------------------------------------

.via:
	call	checkType40		; only cores from CPUID 67xh onwards
	jz	short .none		; have L2 cache
	mov	ecx, 1107h		; check whether L2 cache is disabled
	rdmsr
	test	ax, 100h
	jnz	short .none
	mov	eax, 80000000h		; Via CPUs store L2 cache size
	cpuid				; information in extended CPUID
	cmp	eax, 80000006h		; registers
	jb	short .none
	mov	eax, 80000006h
	cpuid
	shr	ecx, 19			; L2 cache size in bits 31:16 for
	call	checkType44		; Nehemiah, but in 31:24 for
	jz	short .convert		; Samuel 2; others are unclear
	shr	ecx, 8

.convert:				; convert to logarithm as expected by
	xor	al, al			; caller

.loop:
	shr	cx, 1
	jz	short .end
	inc	al
	jmp	short .loop
; ---------------------------------------------------------------------------

.none:
	xor	al, al

.end:
	pop	edx
	pop	ecx
	pop	ebx
	retn


%ifdef PATCH_PENTIUM_PRO

; =============== S U B R O U T I N E =======================================


printECC:
	push	ds
	push	cs
	pop	ds
	mov	eax, 1			; check for Pentium II-based cores
	cpuid				; (without OverDrive processors)
	cmp	ax, 630h
	jb	short .end
	cmp	ax, 660h
	jnb	short .end
	mov	ecx, 11Eh		; check L2 ECC support
	rdmsr
	test	al, 20h
	jz	short .end
	mov	si, aECC
	call	printCtrlString

.end:
	pop	ds
	retn


%ifdef PLATFORM_BX

; =============== S U B R O U T I N E =======================================


writeESCD:
	push	ds
	push	es
	mov	ax, 0
	mov	es, ax
	call	unknown17
	call	unknown18
	call	unknown19
	call	unknown20
	mov	ax, 0F000h
	mov	ds, ax
	mov	di, 56B9h
	mov	eax, 1
	cpuid
	cmp	ax, 630h
	jb	.continue
	mov	al, 0A4h
	mov	dx, 0B2h
	out	dx, al

.continue:
	mov	di, 36B9h
	call	unknown21
	jb	short .writeDone
	push	unknown22
	call	0F000h:dispatchToF000
	jnb	short .writeSuccess
	mov	si, aWriteESCDfail
	call	printString
	jmp	short .writeDone
; ---------------------------------------------------------------------------

.writeSuccess:
	mov	si, aWriteESCDgood
	call	printString

.writeDone:
	call	initMenuChange
	mov	ax, 0
	mov	es, ax
	mov	ax, 0F000h
	mov	ds, ax
	mov	al, byte [es:2008]
	mov	si, unknown23
	mov	[si], al
	call	endMenuChange
	pop	es
	pop	ds
	retn

%endif					; PLATFORM_BX
%endif					; PATCH_PENTIUM_PRO


%if FEATURE_LEVEL = 0

; =============== S U B R O U T I N E =======================================


determineCPUSpeed:
	mov	ax, [bp+cpuFSB]		; get FSB and search the matching
	lea	si, [CPUSpeedLookup]	; lookup table

.nextFSB:
	cmp	ax, [cs:si]
	jnb	short .searchSpeed
	add	si, 4
	jmp	short .nextFSB
; ---------------------------------------------------------------------------

.searchSpeed:
	mov	si, [cs:si+2]		; get speed table
	shr	ax, 2
	add	ax, [bp+cpuFreq]	; get measured CPU speed

.nextSpeed:
	cmp	word [cs:si], 0		; look for best match
	jz	short .end
	cmp	ax, [cs:si]
	jnb	short .found
	add	si, 2
	jmp	.nextSpeed
; ---------------------------------------------------------------------------

.found:
	mov	ax, [cs:si]		; store canonical speed
	mov	[bp+cpuFreq], ax

.end:
	retn

%endif					; FEATURE_LEVEL


; =============== S U B R O U T I N E =======================================


getSMBIOSCPUVer:
					; This logic determines the
					; "Processor Version" and "Processor
					; Family" fields for the SMBIOS
					; structure type 4. It is essentially
					; identical to getCPUBrand.

	mov	al, 01h
	mov	si, aSMBViaC3
	call	checkType4x
	jz	short .end

	mov	al, 0Fh
	mov	si, aSMBCeleron
	call	checkType1
	jz	short .end

	call	checkType4
	jz	short .end

%ifdef PATCH_PENTIUM_PRO
	mov	al, 0Ch
	mov	si, aSMBPentiumPro
	call	checkType9
	jz	short .end

	mov	al, 0Dh
	mov	si, aSMBPentiumOverDr
	call	checkTypeA
	jz	short .end
%endif					; PATCH_PENTIUM_PRO

	mov	al, 11h
	mov	si, aSMBPentium3
	call	checkType5
	jz	short .end

	call	checkType8
	jz	short .checkP3S

	call	checkType6
	jz	short .checkP3Celeron

	jmp	short .default
; ---------------------------------------------------------------------------

.checkP3S:
	push	eax
	call	getL2CacheSize
	cmp	al, 6
	pop	eax
	jnz	short .checkP3Celeron
	mov	si, aSMBPentium3S
	jmp	short .end
; ---------------------------------------------------------------------------

.checkP3Celeron:
	push	eax
	mov	eax, 1
	cpuid
	test	bl, 1
	pop	eax
	jz	short .end
	mov	al, 0Fh
	mov	si, aSMBCeleron
	jmp	short .end
; ---------------------------------------------------------------------------

.default:
	mov	al, 0Dh
	mov	si, aSMBPentium2

.end:
	retn


%if FEATURE_LEVEL = 0

; =============== F U N C T I O N   C H U N K ===============================


callGetSMBIOSCPUVer:
	call	getSMBIOSCPUVer
	mov	[es:di+6], al
	jmp	resumeSMBIOS


; =============== F U N C T I O N   C H U N K ===============================


muCheckVia:
	call	checkType4x
	jz	skipMU
	cmp	si, 0FFFFh
	jmp	resumeMU

%endif					; FEATURE_LEVEL


%if FEATURE_LEVEL <= 1
%ifdef HARDWARE_MONITOR
%ifdef PATCH_MINUS_5V_RAIL

; =============== S U B R O U T I N E =======================================


overrideVRange:
	cmp	ch, 5			; overrides are only for -12 V and
	jb	short .end		; -5 V rails
	cmp	ch, 6			; patch for -5 V: If the reading is
	jnz	short .acceptReading	; the maximum value, we assume that
	mov	bx, 0FF00h		; no -5 V rail is present and simply
	cmp	al, 0FFh		; consider any value admissible.
	jz	short .end

.acceptReading:
%if FEATURE_LEVEL = 1
	call	detectHWMonitor
	jnz	short .W83782D
	mov	bx, 0D7A2h		; -12 V range for W83781D-based chip
%else					; FEATURE_LEVEL
	mov	bx, 0F0A8h
%endif					; FEATURE_LEVEL
	cmp	ch, 6
	jb	short .end
	mov	bx, 0ECB2h		; -5 V range for W83781D-based chip
%if FEATURE_LEVEL = 1
	jmp	short .end
;----------------------------------------------------------------------------

.W83782D:
	mov	bx, 370Fh		; -12 V range for W83782D-based chip
	cmp	ch, 6
	jb	short .end
	mov	bx, 4329h		; -5 V range for W83782D-based chip
%endif					; FEATURE_LEVEL

.end:
	retn

%else					; PATCH_MINUS_5V_RAIL
	overrideVRange	equ	overrideVOrig
%endif					; PATCH_MINUS_5V_RAIL
%endif					; HARDWARE_MONITOR
%endif					; FEATURE_LEVEL
